"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("@stryker-mutator/util");
var plugin_1 = require("stryker-api/plugin");
var tokens_1 = require("./tokens");
var URL_STRYKER_DASHBOARD_REPORTER = 'https://dashboard.stryker-mutator.io/api/reports';
var DashboardReporterClient = /** @class */ (function () {
    function DashboardReporterClient(log, dashboardReporterClient) {
        this.log = log;
        this.dashboardReporterClient = dashboardReporterClient;
    }
    DashboardReporterClient.prototype.postStrykerDashboardReport = function (report) {
        var _this = this;
        var _a;
        this.log.info("Posting report to " + URL_STRYKER_DASHBOARD_REPORTER);
        var reportString = JSON.stringify(report);
        this.log.debug('Posting data %s', reportString);
        return this.dashboardReporterClient.post(URL_STRYKER_DASHBOARD_REPORTER, reportString, (_a = {},
            _a['Content-Type'] = 'application/json',
            _a))
            .then(function (body) {
            var statusCode = body.message.statusCode;
            if (statusCode !== 201) {
                _this.log.error("Post to " + URL_STRYKER_DASHBOARD_REPORTER + " resulted in http status code: " + statusCode + ".");
            }
        })
            .catch(function (err) {
            _this.log.error("Unable to reach " + URL_STRYKER_DASHBOARD_REPORTER + ". Please check your internet connection.", util_1.errorToString(err));
        });
    };
    DashboardReporterClient.inject = plugin_1.tokens(plugin_1.commonTokens.logger, tokens_1.dashboardReporterTokens.httpClient);
    return DashboardReporterClient;
}());
exports.default = DashboardReporterClient;
//# sourceMappingURL=DashboardReporterClient.js.map